import React, { cloneElement } from 'react';
import { TextInput, UIManager, requireNativeComponent } from 'react-native';
import PropTypes from 'prop-types';
import createReactNativeComponentClass from 'react-native/Libraries/Renderer/shims/createReactNativeComponentClass';

export default class MarkedRangeTextInput extends TextInput {
    static displayName = 'MarkedRangeTextInput';

    static propTypes = {
        ...TextInput.propTypes,
        onMarkedRangeChanged: PropTypes.func,
        onChangeText: PropTypes.func,
        shouldReturn: PropTypes.bool
    };

    static defaultProps = {
        shouldReturn: true
    }

    constructor(props) {
        super(props);
        this.clear = this._clear;
    }

    componentWillMount() {
        this._onMarkedRangeChanged = this.onMarkedRangeChanged.bind(this);
        this._onChange = this.onChange.bind(this);
        this._onChangeText = this.onChangeText.bind(this);
    }

    _previousMarkedRange = {};

    _clear() {
        super.clear();
    }

    onChange(event) {
        const { eventCount } = event.nativeEvent;
        // Make sure to fire the mostRecentEventCount first so it is already set on
        // native when the text value is set.
        if (this._inputRef) {
            this._inputRef.setNativeProps({
                mostRecentEventCount: eventCount,
            });
        }

        this.props.onChange && this.props.onChange(event);

        if (!this._inputRef) {
            // calling `this.props.onChange` or `this.props.onChangeText`
            // may clean up the input itself. Exits here.
            return;
        }

        this.forceUpdate();
    }

    onChangeText(event) {
        const { onChangeText } = this.props;
        const { nativeEvent: { text: text } } = event;
        onChangeText && onChangeText(text);
        this.forceUpdate();
    }

    onMarkedRangeChanged(event) {
        const { onMarkedRangeChanged } = this.props;

        const previousMarkedRange = this._previousMarkedRange;
        const { nativeEvent: { markedRange: { start, end, markedText, text } } } = event;

        if (previousMarkedRange.start === start &&
            previousMarkedRange.end === end &&
            previousMarkedRange.text === text) {
            return;
        }

        onMarkedRangeChanged && onMarkedRangeChanged(event);
        this._previousMarkedRange = { start, end, markedText, text };
        this.forceUpdate();
    }

    render() {
        const wrapper = super.render();
        const textInput = wrapper.props.children;
        const props = {
            ...textInput.props,
            ref: this._setNativeRef,
            onChange: this._onChange,
            onChangeText: this._onChangeText,
            onMarkedRangeChanged: this._onMarkedRangeChanged
        };

        let NativeTextInput = RNMarkedRangeTextField;

        if (this.props.multiline) {
            NativeTextInput = RNMarkedRangeTextView;
        }

        return cloneElement(wrapper, wrapper.props, <NativeTextInput {...props} />);
    }
}

const RNMarkedRangeTextField = requireNativeComponent('RNMarkedRangeTextField', null);
const RNMarkedRangeTextView = requireNativeComponent('RNMarkedRangeTextView', null);
